# encoding: UTF-8
import os, os.path
import csv
import nltk
from nltk.corpus import stopwords
import re
import logging
import cPickle

import generate_corpus
from utils import BlogLabels

logger = logging.getLogger(__name__)

try:
    my_stopwords = stopwords.words("english")
except LookupError:
    nltk.download("stopwords")    
    my_stopwords = stopwords.words("english")

try:
    nltk.pos_tag([])
except LookupError:
    nltk.download("maxent_treebank_pos_tagger") # may change if pos_tag's recommended tagging method changes


my_stopwords = set(my_stopwords).union(
        # add contractions created by TreebankWordTokenizer
        ("'s", "'ll", "'re","'ve","n't", "'m", "'d")).union(
        # add some artefacts
        ("\\", "\\'", '\\"', "'", '"', "(", ")",
         "&amp", "&raquo")).union(
        # add own words
        ("go", "make", 
         "ad", "alway",
         "new", "recent", 
         "recommend", "someth", 
         "weblog", "blog",)
    )

stemmer = nltk.PorterStemmer()

unwanted_tags = set(("''", "``", 
                    '(', ')', ',', '--', '.', ':', 
                    'CC', 
                    'CD', # numeral - maybe feature count of CD
                    'DT',
                    'EX',
                    'IN',
                    'LS',
                    'MD',
                    'PDT',
                    'POS',
                    'PRP',
                    'PRP$',
                    'RP',
                    'SYM',
                    'NNP',
                    'TO',
                    'UH',
                    'WDT',
                    'WP',
                    'WP$',
                    'WRB',
                    'JJR'))

def word_splitter(words):
    """splits words by hyphen.
    """
    for word in words:
        parts = word.split("-")
        for part in parts:
            yield part

def extract_features(text):
    """Extracts words counts. All words are lowercase."""
    tokens = nltk.word_tokenize(text)
    # remove ellipsis
    tokens = (t if not t.endswith("..") else t[:-2] for t in tokens)
    
    tokens = [word for word in word_splitter(tokens) if 
                                len(word) > 2 and
                                "/" not in word and
                                "." not in word and
                                "," not in word and
                                "&" not in word and
                                "'" not in word and
                                not word[0].isdigit()]
    
    tagged = nltk.pos_tag(tokens)
    informative_words = (word.lower() for word, tag in tagged 
                            if tag not in unwanted_tags)

    stemmed = (stemmer.stem(w) for w in informative_words)
    
    stop_filtered = (word for word in stemmed if word not in my_stopwords)
    
    fd = nltk.FreqDist(stop_filtered)
    
    return fd

def from_files_generator(corpus_folder):
    for name in os.listdir(corpus_folder):
        with open(os.path.join(corpus_folder, name)) as f:
            text = f.read().decode("utf-8")
        yield (name, 1, text)

def write_feature_matrix(filename, features, names):
    sections = BlogLabels("feeds_urls.edited")
    names = ["BLOG", "SECTION"] + sorted(list(names))
    with open(filename, "w") as out:
        writer = csv.DictWriter(out, fieldnames=names, restval=0)
        writer.writerow(dict((field, field) for field in names))
        for blog, freqs in features.iteritems():
            freqs["BLOG"] = blog
            freqs["SECTION"] = sections[blog]
            writer.writerow(freqs)

def extract_from_generator(generator):
    """ Extracts features from generator returning tuples:
    (observation_id, section_weight, text)
    
    returns (extracted_features, feature_names)
    
    extracted_features = { "www.blog.com" : {"my" : 30, "ego": 14 ...}, ... }
    feature_names = set("my", "ego", "doctor", ...)
    """
    extracted_features = {}
    feature_names = set()
    for observation_id, section_weight, text in generator:
        logger.info(observation_id)
        new_features = dict(extract_features(text))
        current_features = extracted_features.setdefault(observation_id, 
                                                        {})
        
        for word, count in new_features.iteritems():
            current_features[word] = section_weight * count + current_features.get(word, 0)
        
        feature_names = feature_names.union(new_features.keys())
    
    return extracted_features, feature_names

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    #    features, names = extract_from_generator(from_files_generator(corpus_folder))
    features, names = extract_from_generator(generate_corpus.from_feeds_texts())
    print "%d names found" % len(names)
    logger.debug("\n".join(names))
    
    write_feature_matrix("feature_matrix.csv", features, names)
