# encoding: utf-8
"""
Writes utf-8 encoded document texts to corpus folder.
"""
import os.path
import feedparser
import shutil
import errno
import logging
from BeautifulSoup import BeautifulSoup
import nltk

logger = logging.getLogger(__name__)

CORPUS_FOLDER = "corpus"
FEEDS_FOLDER = "feeds"

TITLE_WEIGHT = 5
SUBTITLE_WEIGHT = 5
ENTRY_SUMMARY_WEIGHT = 1
ENTRY_TITLE_WEIGHT = 4
CONTENT_WEIGHT = 1

def iterate_over_folder(folder, callback):
    """
    callback(filename, text)
    """
    for name in (n for n in os.listdir(folder) if not os.path.isdir(n)):
        with open(os.path.join(folder, name)) as f:
            body = f.read()
        callback(name, body)

def clear_directory(path):
    try:
        shutil.rmtree(path)
        os.mkdir(path)
    except OSError, e:
        if e.errno != errno.ENOENT: # doesn't exist
            raise

def write_corpus(generator):
    """writes utf-8 encoded files to corpus folder"""
    clear_directory(CORPUS_FOLDER)
    for name, section_weight, text in generator:
        with open(os.path.join(CORPUS_FOLDER, name), "a") as out:
            if isinstance(text, unicode):
                text = text.encode("utf-8")
            out.write(text)
            out.write("\n")
            
def from_feeds_descriptions():
    for name in (n for n in os.listdir(FEEDS_FOLDER) 
                    if not os.path.isdir(n)):
        feed = feedparser.parse(os.path.join(FEEDS_FOLDER, name))
        text = ""
        try:
            text = extract_subtitle_from_feed(feed)
        except ValueError, e:
            logger.warning("%s (%s)" % (e, name))
        
        yield name, 1, text

def from_feeds_texts():
    for name in (n for n in os.listdir(FEEDS_FOLDER) 
                    if not os.path.isdir(n)):
        feed = feedparser.parse(os.path.join(FEEDS_FOLDER, name))
        try:
            text = extract_subtitle_from_feed(feed)
            yield name, SUBTITLE_WEIGHT, text
        except ValueError, e:
            logger.warning("%s (%s)" % (e, name))
            
        try:
            text = extract_field_from_feed(feed, "title")
            yield name, TITLE_WEIGHT, text
        except ValueError, e:
            logger.warning("%s (%s)" % (e, name))

        for entry in feed["entries"]:
            try:
                yield name, ENTRY_SUMMARY_WEIGHT, get_text_from_detail(entry["summary_detail"])
            except KeyError:
                logger.debug("No summary in %s" % name)
                assert "summary" not in entry
            
            try:
                text = get_text_from_detail(entry["title_detail"])
                yield name, ENTRY_TITLE_WEIGHT, text
            except KeyError:
                logger.info("No title for entry in %s" % name)
            try:
                for content in entry["content"]:
                    text = get_text_from_detail(content)
                    yield name, CONTENT_WEIGHT, text
            except KeyError:
                logger.debug("No content in %s" % name) # hey, just trust feedparser that content has allways type and value
            

def get_text_from_detail(content):
    if "html" in content["type"]:
        try:
            text_with_entities = nltk.clean_html(content["value"])
            return unicode(BeautifulSoup(text_with_entities, 
                    convertEntities=BeautifulSoup.HTML_ENTITIES))
        except ValueError, e:
            logger.error("%s \ntext: %s" % (e, content["value"]))
    else:
        return content["value"]

def extract_field_from_feed(feed, field):
    try:
        subtitle = feed["feed"][field].strip()
    except KeyError:
        subtitle = ""
    
    if subtitle == "":
        raise ValueError("no %s in feed" % field)
    
    return subtitle    

def extract_subtitle_from_feed(feed):
    return extract_field_from_feed(feed, "subtitle")
        
if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    write_corpus(from_feeds_texts())
